/* valaemptystatement.vala
 *
 * Copyright (C) 2006  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */
#define VALA_FREE_CHECKED(o,f) ((o) == NULL ? NULL : ((o) = (f (o), NULL)))
#define VALA_FREE_UNCHECKED(o,f) ((o) = (f (o), NULL))

#include "valaemptystatement.h"
#include <vala/valasourcereference.h>
#include <vala/valacodevisitor.h>
#include <vala/valaemptystatement.h>

struct _ValaEmptyStatementPrivate {
};
#define VALA_EMPTY_STATEMENT_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_EMPTY_STATEMENT, ValaEmptyStatementPrivate))
enum  {
	VALA_EMPTY_STATEMENT_DUMMY_PROPERTY,
};
static void vala_empty_statement_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor);
static gpointer vala_empty_statement_parent_class = NULL;


/**
 * Creates a new empty statement.
 *
 * @param source reference to source code
 * @return       newly created empty statement
 */
ValaEmptyStatement* vala_empty_statement_new (ValaSourceReference* source)
{
	g_return_val_if_fail (source == NULL || VALA_IS_SOURCE_REFERENCE (source), NULL);
	GParameter * __params = g_new0 (GParameter, 1);
	GParameter * __params_it = __params;
	(__params_it->name = "source-reference", g_value_init (&__params_it->value, VALA_TYPE_SOURCE_REFERENCE), g_value_set_object (&__params_it->value, source), __params_it++);
	ValaEmptyStatement * self = g_object_newv (VALA_TYPE_EMPTY_STATEMENT, __params_it - __params, __params);
	return self;
}


static void vala_empty_statement_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor)
{
	ValaEmptyStatement * self = VALA_EMPTY_STATEMENT (base);
	g_return_if_fail (VALA_IS_CODE_VISITOR (visitor));
	vala_code_visitor_visit_empty_statement (visitor, self);
}


static void vala_empty_statement_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	ValaEmptyStatement * self = VALA_EMPTY_STATEMENT (object);
	switch (property_id) {
	}
}


static void vala_empty_statement_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	ValaEmptyStatement * self = VALA_EMPTY_STATEMENT (object);
	switch (property_id) {
	}
}


static void vala_empty_statement_class_init (ValaEmptyStatementClass * klass)
{
	vala_empty_statement_parent_class = g_type_class_peek_parent (klass);
	G_OBJECT_CLASS (klass)->get_property = vala_empty_statement_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_empty_statement_set_property;
	VALA_CODE_NODE_CLASS (klass)->accept = vala_empty_statement_real_accept;
}


static void vala_empty_statement_init (ValaEmptyStatement * self)
{
}


GType vala_empty_statement_get_type ()
{
	static GType vala_empty_statement_type_id = 0;
	if (G_UNLIKELY (vala_empty_statement_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaEmptyStatementClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_empty_statement_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaEmptyStatement), 0, (GInstanceInitFunc) vala_empty_statement_init };
		vala_empty_statement_type_id = g_type_register_static (VALA_TYPE_STATEMENT, "ValaEmptyStatement", &g_define_type_info, 0);
	}
	return vala_empty_statement_type_id;
}




