/* valastatement.vala
 *
 * Copyright (C) 2006  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */
#define VALA_FREE_CHECKED(o,f) ((o) == NULL ? NULL : ((o) = (f (o), NULL)))
#define VALA_FREE_UNCHECKED(o,f) ((o) = (f (o), NULL))

#include "valastatement.h"
#include <vala/valastatement.h>

struct _ValaStatementPrivate {
	gboolean _construction;
};
#define VALA_STATEMENT_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_STATEMENT, ValaStatementPrivate))
enum  {
	VALA_STATEMENT_DUMMY_PROPERTY,
	VALA_STATEMENT_CONSTRUCTION,
};
static gint vala_statement_real_get_number_of_set_construction_parameters (ValaStatement* self);
static gpointer vala_statement_parent_class = NULL;
static void vala_statement_dispose (GObject * obj);


/**
 * Returns the number of construction parameters this statement sets in
 * maximum or -1 if this statement may not be used in the construction
 * part of a construction method.
 *
 * @return number of construction parameters set or -1
 */
static gint vala_statement_real_get_number_of_set_construction_parameters (ValaStatement* self)
{
	g_return_val_if_fail (VALA_IS_STATEMENT (self), 0);
	return -1;
}


gint vala_statement_get_number_of_set_construction_parameters (ValaStatement* self)
{
	return VALA_STATEMENT_GET_CLASS (self)->get_number_of_set_construction_parameters (self);
}


gboolean vala_statement_get_construction (ValaStatement* self)
{
	g_return_val_if_fail (VALA_IS_STATEMENT (self), FALSE);
	return self->priv->_construction;
}


void vala_statement_set_construction (ValaStatement* self, gboolean value)
{
	g_return_if_fail (VALA_IS_STATEMENT (self));
	self->priv->_construction = value;
}


static void vala_statement_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	ValaStatement * self = VALA_STATEMENT (object);
	switch (property_id) {
		case VALA_STATEMENT_CONSTRUCTION:
		g_value_set_boolean (value, vala_statement_get_construction (self));
		break;
	}
}


static void vala_statement_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	ValaStatement * self = VALA_STATEMENT (object);
	switch (property_id) {
		case VALA_STATEMENT_CONSTRUCTION:
		vala_statement_set_construction (self, g_value_get_boolean (value));
		break;
	}
}


static void vala_statement_class_init (ValaStatementClass * klass)
{
	vala_statement_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (ValaStatementPrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_statement_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_statement_set_property;
	G_OBJECT_CLASS (klass)->dispose = vala_statement_dispose;
	VALA_STATEMENT_CLASS (klass)->get_number_of_set_construction_parameters = vala_statement_real_get_number_of_set_construction_parameters;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_STATEMENT_CONSTRUCTION, g_param_spec_boolean ("construction", "foo", "bar", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
}


static void vala_statement_init (ValaStatement * self)
{
	self->priv = VALA_STATEMENT_GET_PRIVATE (self);
}


static void vala_statement_dispose (GObject * obj)
{
	ValaStatement * self = VALA_STATEMENT (obj);
	ValaStatementClass * klass;
	GObjectClass * parent_class;
	klass = VALA_STATEMENT_CLASS (g_type_class_peek (VALA_TYPE_STATEMENT));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType vala_statement_get_type ()
{
	static GType vala_statement_type_id = 0;
	if (G_UNLIKELY (vala_statement_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaStatementClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_statement_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaStatement), 0, (GInstanceInitFunc) vala_statement_init };
		vala_statement_type_id = g_type_register_static (VALA_TYPE_CODE_NODE, "ValaStatement", &g_define_type_info, G_TYPE_FLAG_ABSTRACT);
	}
	return vala_statement_type_id;
}




